//*******************************************************************************************
// Projeto - ADR-1 1/2 - Adaptação do projeto ADR-1 desenvolvido com BASIC STAMP p/ Arduíno
//
// Desenvolvido por: Eng. Márcio José Soares
// versão: 1.0 de 24/02/2021
//
// Plataforma - Arduino Pro Mini - ATMEGA328 - 5VDC - 16MHz
// Usa sensor IR Sharp 2Y0A21
// Usa dois servos Futaba FP-S148 adaptados para giro
// Usa um módulo RX315MHz com HT12D 
//
//*******************************************************************************************
//
// Pinos utilizados:
// 10 - SERVO1
// 11 - SERVO2
// 13 - LEDLive
//  4 - BUZZER
// A0 - entrada do Sensor IR
//  6 - D0 do HT12D
//  7 - D1 do HT12D
//  8 - D2 do HT12D
//  9 - D3 do HT12D
//*******************************************************************************************
// Últimas atualizações:
//    em 13/04/2021:
//      - criada essa versão baseada na versão ADR-1-1.5_arduino.ini
//
//*******************************************************************************************

//********************************************************************************
// Arquivos incluídos no módulo
//********************************************************************************
#include "pindeclarations.h"
#include <Servo.h>
#include <SharpIR.h>

//********************************************************************************
// Definições para pinos de I/O
//********************************************************************************
#define LEDLive     PB5     //13
#define SRV1        PB2     //10
#define SRV2        PB3     //11
#define IRPin       A0
#define pinBuzzer   PD4     //4

#define pinD0       PD6     //6
#define pinD1       PD7     //7
#define pinD2       PB0     //8
#define pinD3       PB1     //9

//********************************************************************************
// Definições importantes do módulo
//********************************************************************************
/* Model :
  GP2Y0A02YK0F --> 20150
  GP2Y0A21YK0F --> 1080
  GP2Y0A710K0F --> 100500
  GP2YA41SK0F --> 430
*/
#define model 1080      //

#define TRUE      1
#define FALSE     0

#define FWD       250
#define STOP1     73
#define STOP2     73
#define RWD       0

#define CFWD     1        //para frente
#define CRWD     2        //para tras
#define CRIGHT   4        //para direita
#define CLEFT    8        //para esquerda
#define CSTOP    0        //para stop
#define FSRIGHT  5        //para frente e direita juntos
#define FSLEFT   9        //para frente e esquerda juntos
#define RSRIGHT  6        //para tras e direita juntos
#define RSLEFT   10       //para tras e esquerda juntos

#define buzzerON  (digitalWrite(pinBuzzer,HIGH))
#define buzzerOFF (digitalWrite(pinBuzzer,LOW))

//********************************************************************************
// Variáveis globais do módulo
//********************************************************************************
int LEDliveState = LOW;
long interval = 1;                 //ms  - tempo para intervalo
long previous = 0;

Servo ADR1Servos[2];
int servoPins[2] = {10,11};
int SRVPosition[3] = {90,90,90};
//long duration, cm;
int distancia;
  
//********************************************************************************
// Constantes globais do módulo
//********************************************************************************
const int LEDTIME = 250;           //ms  - 250 x 2ms  = 500ms
const int TIMEOUT = 4;             //ms  - 4 x 500ms = 2s
const int INSTIME = 125;           //ms  - 125 x 2ms = 250ms

//********************************************************************************
//Inicializa o sensor nos pinos do IR Sharp
SharpIR mySensor = SharpIR(IRPin, model);

//********************************************************************************
// Setup - Arduino
//********************************************************************************
void setup() {
    //configura porta para debug - acompanhamento das passagens
    Serial.begin(2400);

    //define direção dos pinos
    pinMode(SRV1, OUTPUT);
    pinMode(SRV2, OUTPUT);
    pinMode(LEDLive,OUTPUT);
    pinMode(pinBuzzer,OUTPUT);
    
    pinMode(pinD0, INPUT);
    pinMode(pinD1, INPUT);
    pinMode(pinD2, INPUT);
    pinMode(pinD3, INPUT);
}

//********************************************************************************
// Loop - Arduino
//********************************************************************************
void loop() {
    unsigned long current = 0;
    uint16_t timeLED = 0;
    uint16_t timeINs = 0;
    uint16_t t_500ms = 0;
    uint8_t myins = 0;
    int val = 0;
    
    buzzerOFF;          //buzzer desligado!
    initservos();       //inicia servos
    move_STOP();        //pará robô
    
    digitalWrite(LEDLive, LEDliveState);     //LED Live apagado
    delay(1000);

    while(TRUE)
    {
         
        current = millis();                       //tempo atual em ms
        if((current - previous) > interval)
        {
            previous = current;                   //salva tempo atual
     
            timeLED++;
            if(timeLED == LEDTIME)
            {
                timeLED = 0;  
            
                if(LEDliveState == LOW)           //troca estado atual do LED
                {
                    LEDliveState = HIGH;
                }
                else
                {
                    LEDliveState = LOW;
                }
            
                digitalWrite(LEDLive, LEDliveState);  
                read_IRSharp();
            }

            timeINs++;
            if(timeINs == INSTIME)
            {
                timeINs = 0;
                
                val = digitalRead(pinD0);                 //lê pinos
                if(val == HIGH)
                    myins |= 0x01;
                else
                    myins &= ~0x01;

                val = digitalRead(pinD1);                 //lê pinos
                if(val == HIGH)
                    myins |= 0x02;
                else
                    myins &= ~0x02;


                val = digitalRead(pinD2);                 //lê pinos
                if(val == HIGH)
                    myins |= 0x04;
                else
                    myins &= ~0x04;


                val = digitalRead(pinD3);                 //lê pinos
                if(val == HIGH)
                    myins |= 0x08;
                else
                    myins &= ~0x08;


                trata_comm(myins);
            }
        }
    }
}

//****************************************************************************
// Função para mover o robô após comando serial recebido
//
// Entradas - pinos de entrada com captura dos dados do HT12D
// Saídas   - nenhuma
//****************************************************************************
void trata_comm(uint8_t myins){

    switch(myins){            //testa comando
        case CFWD:
            move_FWD();       //move para frente
            break;
        case CRWD:
            move_RWD();
            break;
        case CLEFT:
            turn_LEFT();
            break;
       case CRIGHT:
            turn_RIGHT();
            break;
        case FSLEFT:    
            spin_LEFT();
            break;
        case FSRIGHT:
            spin_RIGHT();
            break;   
        case RSLEFT:    
            spin_RIGHT();
            break;
        case RSRIGHT:
            spin_LEFT();
            break;     
        case CSTOP:
            move_STOP();
            break;
        default:
            move_STOP();
    }
}

//****************************************************************************
// Funcção para ler o sensor IR Sharp 2Y0A21
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void read_IRSharp(void){

    distancia = mySensor.distance();    //pega a distância
    /*Serial.print("Dist = ");
    Serial.print(distancia);
    Serial.println(" cm");*/

    if(distancia <= 10){              //se distância menor/igual a 10cm
        buzzerON;                     //liga buzzer
        move_STOP();                  //para o robô  
        move_RWD();                   //move para trás
        delay(2000);                  //por 3 segundos
        spin_LEFT();                  //gira a esquerda
        delay(500);                  //por 3 segundos
        move_STOP();                  //para
        buzzerOFF;                    //desliga buzzer
    }

}

//****************************************************************************
// Função para iniciar os servos
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void initservos(void){
    int i;
    for(i=0; i<2; i++){
        ADR1Servos[i].attach(servoPins[i]);
    }
    move_STOP();
}

//****************************************************************************
// Função para parar
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void move_STOP(void){
    ADR1Servos[0].write(STOP1);
    ADR1Servos[1].write(STOP2);
}

//****************************************************************************
// Função para andar para frente
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void move_FWD(void){

    ADR1Servos[0].write(FWD);
    ADR1Servos[1].write(FWD);
    
}

//****************************************************************************
// Função para andar para trás
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void move_RWD(void){

    ADR1Servos[0].write(RWD);
    ADR1Servos[1].write(RWD);
    
}

//****************************************************************************
// Função para virar para esquerda - move apenas 1 servo
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void turn_LEFT(void){

    ADR1Servos[0].write(STOP1);
    ADR1Servos[1].write(FWD);
    
}

//****************************************************************************
// Função para girar para esquerda - move ambos os servos (a frente)
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void spin_LEFT(void){

    ADR1Servos[0].write(RWD);
    ADR1Servos[1].write(FWD);
}

//****************************************************************************
// Função para virar para direita - move apenas 1 servo
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void turn_RIGHT(void){

    ADR1Servos[0].write(FWD);
    ADR1Servos[1].write(STOP2);
}

//****************************************************************************
// Função para girar para direita - move ambos os servos
//
// Entradas - nenhuma
// Saídas   - nenhuma
//****************************************************************************
void spin_RIGHT(void){

    ADR1Servos[0].write(FWD);
    ADR1Servos[1].write(RWD);
}
